// C Source File
// Created 9/20/03; 8:48:59 AM

#include <tigcclib.h>         // Include All Header Files
#include "screen.h"
#include "shipdesigns.h"
#include "shipmath.h"
#include "ship.h"
#include "utils.h"
#include "bomb.h"
#include "explosion.h"
#include "packets.h"
#include "extgraph.h"


typedef enum {LINK, AI} GAMEMODE;
typedef enum {SERIES, CUSTOM} AIMODE;

float game(SCREEN_DIM dimensions, GAMEMODE gamemode, AIMODE aimode);
void keyMovement(SHIP *ship);
void AImovement(SHIP *ship);

// Main Function
void _main(void)
{
	//set up all instance variables, create environment, preliminary functions
	
	char buf[8];
	GAMEMODE mode;
	AIMODE aimode;

	int key;

	int linkid = -1;

	mainmenu:

	ClrScr();
	
	FontSetSys(F_6x8);
	
	DrawStr(0,0,"Vector Space 0.5.1 beta",A_NORMAL);
	DrawStr(0,10,"By Paulygon Software",A_NORMAL);
	DrawStr(0,20,"http://paulygon.org",A_NORMAL);

	DrawStr(0,70,"Press 1: Custom Game",A_NORMAL);
	DrawStr(0,80,"      2: Full Game",A_NORMAL);
	DrawStr(0,90,"      3: Human vs. Link",A_SHADED);
	
	SCREEN_DIM dimensions;
	
	key = ngetchx();
	
	if (key == KEY_ESC)
		return;

	if (TI89)
		dimensions = SetDimensions(	area_width/2-80,area_height/2-50,
																area_width/2+80,area_height/2+50	);
	else
		dimensions = SetDimensions(	area_width/2-120,area_height/2-64,
																area_width/2+120,area_height/2+64	);
	
	if (key == '1')
	{
		mode = AI;
		aimode = CUSTOM;
	}
	else if (key == '2')
	{
		mode = AI;
		aimode = SERIES;
	}
	else {
		goto mainmenu;
	}
	
	int fps = game(dimensions, mode, aimode);
	/*
	ClrScr();
	FontSetSys(F_6x8);
	sprintf(buf, "%i", fps);
	DrawStr(0,0,"Frames/Second:",A_NORMAL);
	DrawStr(0,10,buf,A_NORMAL);
	DrawStr(0,50,"(Press ENTER)",A_NORMAL);
	
	while (ngetchx() != KEY_ENTER);*/
}

float game(SCREEN_DIM dimensions, GAMEMODE gamemode, AIMODE aimode)
{

	char *screen = malloc(GRAYDBUFFER_SIZE);
	if(screen == NULL) return 0; //not enough memory
	//PortSet (screen, 239, 127);

	if (!GrayOn())
		return 0;
		
	GrayDBufInit(screen);
	GrayDBufSetActiveIdxSync(1);

	short *shipdesign;
	short shiplen;

	short *aidesign;
	short ailen;

	int shipid;

	SHIP player;
	SHIP aiplayer;
	
	player = chooseship(screen, &shipid, "your");
	shipdesign = getdesign(shipid, &shiplen);
	
	if (aimode==CUSTOM)
	{
		aiplayer = chooseship(screen, &shipid, "the opponent");
		aidesign = getdesign(shipid, &ailen);
	}
		
	player.angle = 0;

	SHIP linkPlayer;

	if (gamemode == LINK)
	{
		ClrScr();
	
		char linkdata;
	
		int quit = 0;
		int linkid=-1;
	
		linkdata = 100+shipid;
		DrawStr(0,0,"Pinging...", A_NORMAL);
		LCD_restore(screen);
		LIO_SendData(&linkdata, sizeof(char));
		
		char charbuffer[26];
		
		int opponentid;
		
		DrawStr(0,8,"Waiting for reply...", A_NORMAL);
		LCD_restore(screen);
		if (!LIO_RecvData(&linkdata,sizeof(char),20))
		{
			if (linkdata <= 100) {
				quit=1;
			}
			else {
				opponentid = linkdata-100;
				linkid = 2;
				LIO_SendData(&player, sizeof(SHIP));
				LIO_RecvData(&linkPlayer, sizeof(SHIP), 0);
			}
		}	
		else {
			linkid = 1;			
			DrawStr(0,16,"Waiting for other calc...", A_NORMAL);
			LCD_restore(screen);
			int error = LIO_RecvData(&linkdata,sizeof(char),0);
			switch(error)
			{
				case 0:
					if (linkdata > 100) {
						opponentid = linkdata-100;
						linkdata = 100+shipid;
						LIO_SendData(&linkdata, sizeof(char));
						LIO_RecvData(&linkPlayer, sizeof(SHIP), 0);
						LIO_SendData(&player, sizeof(SHIP));
					}
					else
						quit = 1;
	
					break;
				
				case 1:
					DrawStr(0,24,"ERR:No cable connected!", A_NORMAL);
					LCD_restore(screen);
					ngetchx();
					quit = 1;
					break;
					
				default:
					sprintf(charbuffer, "ERR: Code %d", error);
					LCD_restore(screen);
					DrawStr(0,24,charbuffer,A_NORMAL);
					ngetchx();
					quit = 1;
			}
		}
		

		aidesign = getdesign(opponentid, &ailen);

		if (quit==1) {
			PortRestore();
			free(screen);
			return 0;
		}

		if (linkid == 1) {
			player.x = 20;
			player.y = 20;
			linkPlayer.x = area_width-20;
			linkPlayer.y = area_height-20;
		}
		if (linkid == 2) {
			player.x = area_width-20;
			player.y = area_height-20;
			linkPlayer.x = 20;
			linkPlayer.y = 20;
		}
	}


	
	int i;

	char done = 0;

	if(gamemode == AI) {
		player.x = area_width-20;
		player.y = area_height-20;
		player.angle = 225;

//		aidesign = getdesign(0, &ailen);
		aiplayer.x = 20;
		aiplayer.y = 20;
		aiplayer.angle = 0;
/*		aiplayer.radius = 12;
		aiplayer.speed = 4;
		aiplayer.energy = 50;
		aiplayer.energyregen = 2;
		aiplayer.bombloss = 1;
		aiplayer.bombdamage = 2;
		aiplayer.safe = 0;*/
		aiplayer.bombdamage++;
		if (aiplayer.bombloss > 1)
			aiplayer.bombloss--;
	}
	else
	{
		aiplayer = linkPlayer;
	}
	
	
	player.numBombs = 0;
	aiplayer.numBombs = 0;


	EXPLOSION explosion[MAX_BOMBS*2];
	short numExplosions = 0;

	OSFreeTimer (USER_TIMER);
	OSRegisterTimer (USER_TIMER, 20);

	int frames = 0, seconds = 0;

	short tempangle;

	OSFreeTimer(MISC_TIMER);
	OSRegisterTimer(MISC_TIMER, 2);

	short screen_centerX = LCD_WIDTH/2;

	char won = -1;
	char shoot = 0;

	char string[32];
		
	int tx, ty;

	double mapfactor = .1;
	WIN_RECT mapRect;
	mapRect.x0 = 0;
	mapRect.y0 = LCD_HEIGHT-area_height*mapfactor-1;
	mapRect.x1 = area_width*mapfactor+mapRect.x0;
	mapRect.y1 = area_height*mapfactor+mapRect.y0;

	short temp;
	
	int safetyAngle = 0;

	PACKET newpacket;
	newpacket.enabled = 0;
	newpacket.angle = 0;

	GKeyFlush();
	
	randomize();
	
	int reset = 1, aiplayerid = 0, destroyed = 0;;

	while (!done && (won == -1 || numExplosions != 0)) {
		
		if (reset && aimode == SERIES && gamemode == AI)
		{
			aiplayerid++;
			if (aiplayerid > MAX_SHIPS)
				aiplayerid = 1;
				
			aidesign = getdesign(aiplayerid, &ailen);
			getshipsettings(&aiplayer, aiplayerid);
			aiplayer.x = 20;
			aiplayer.y = 20;
			aiplayer.bombdamage++;
			if (aiplayer.bombloss > 1)
				aiplayer.bombloss--;
			
			reset = 0;
		}
		
		//draw screen (buffer):

		ClearGrayScreen2B(GrayDBufGetPlane(GrayDBufGetHiddenIdx(), LIGHT_PLANE), GrayDBufGetPlane(GrayDBufGetHiddenIdx(), DARK_PLANE));

		FontSetSys(F_4x6);

		GrayDBufSetHiddenAMSPlane(DARK_PLANE);
		
		if (player.energy > 0) {
			DrawLine(LCD_WIDTH-player.energy-6,3,LCD_WIDTH-6,3,A_NORMAL);
			DrawLine(LCD_WIDTH-player.energy-6,4,LCD_WIDTH-6,4,A_NORMAL);
		}
		DrawStr(LCD_WIDTH-5, 1, "E", A_NORMAL);

		DrawLine(LCD_WIDTH-(player.energyregen*3)-6,10,LCD_WIDTH-6,10,A_NORMAL);
		DrawLine(LCD_WIDTH-(player.energyregen*3)-6,11,LCD_WIDTH-6,11,A_NORMAL);
		DrawStr(LCD_WIDTH-5, 8, "R", A_NORMAL);

		DrawLine(LCD_WIDTH-(player.bombdamage*3)-6,17,LCD_WIDTH-6,17,A_NORMAL);
		DrawLine(LCD_WIDTH-(player.bombdamage*3)-6,18,LCD_WIDTH-6,18,A_NORMAL);
		DrawStr(LCD_WIDTH-5, 15, "D", A_NORMAL);

		DrawLine(LCD_WIDTH-(player.speed*2)-6,24,LCD_WIDTH-6,24,A_NORMAL);
		DrawLine(LCD_WIDTH-(player.speed*2)-6,25,LCD_WIDTH-6,25,A_NORMAL);
		DrawStr(LCD_WIDTH-5, 22, "S", A_NORMAL);

		DrawLine(LCD_WIDTH-(10/player.shielding)-6,31,LCD_WIDTH-6,31,A_NORMAL);
		DrawLine(LCD_WIDTH-(10/player.shielding)-6,32,LCD_WIDTH-6,32,A_NORMAL);
		DrawStr(LCD_WIDTH-5, 29, "H", A_NORMAL);

		
		DrawClipRect(&mapRect, &(SCR_RECT){{0, 0, LCD_WIDTH-1, LCD_HEIGHT-1}}, A_NORMAL);
		DrawPix(player.x*mapfactor+mapRect.x0, player.y*mapfactor+mapRect.y0,A_NORMAL);
		DrawPix(aiplayer.x*mapfactor+mapRect.x0, aiplayer.y*mapfactor+mapRect.y0,A_NORMAL);

		if (newpacket.enabled)
			DrawPix(newpacket.x*mapfactor+mapRect.x0, newpacket.y*mapfactor+mapRect.y0,A_NORMAL);

		for (i=0; i<player.numBombs; i++)
			DrawBomb(&dimensions, &player.bomb[i]);

		for (i=0; i<aiplayer.numBombs; i++)
			DrawBomb(&dimensions, &aiplayer.bomb[i]);

		DrawClipEllipse(mapRect.x0+(area_width/2)*mapfactor,mapRect.y0+(area_height/2)*mapfactor,mapfactor*32,mapfactor*32,ScrRect, A_NORMAL);

		packetStuff(&dimensions, &newpacket);

		if (player.energy > 0) DrawShip(player.x, player.y, player.radius, player.angle, &dimensions, shipdesign, shiplen);
		if (aiplayer.energy > 0) DrawShip(aiplayer.x, aiplayer.y, aiplayer.radius, aiplayer.angle, &dimensions, aidesign, ailen);

		
		for (i=0; i<numExplosions; i++)
			DrawExplosion(&dimensions, &explosion[i]);

		GrayDBufSetHiddenAMSPlane(LIGHT_PLANE);
		if (player.safe)
			DrawStr(0,0,"Safe",A_NORMAL);

		tx = aiplayer.x - dimensions.a_x + 12;
		ty = aiplayer.y - dimensions.a_y + 12;

		if (tx >= 0 && tx < LCD_WIDTH &&
				ty >= 0 && ty < LCD_HEIGHT && aiplayer.energy > 0) {
			FontSetSys(F_4x6);					
			sprintf(string,"%d",aiplayer.energy);
			DrawStr(tx, ty, string, A_NORMAL);
		}
		
		DrawBorders(&dimensions);
		
		DrawPix(player.x*mapfactor+mapRect.x0, player.y*mapfactor+mapRect.y0,A_NORMAL);
		DrawPix(aiplayer.x*mapfactor+mapRect.x0, aiplayer.y*mapfactor+mapRect.y0,A_NORMAL);


		DrawClipRect(&(WIN_RECT){dimensions.a_x*mapfactor+mapRect.x0, dimensions.a_y*mapfactor+mapRect.y0, dimensions.b_x*mapfactor+mapRect.x0, dimensions.b_y*mapfactor+mapRect.y0},
									&(SCR_RECT){{0, 0, LCD_WIDTH-1, LCD_HEIGHT-1}},
									A_NORMAL);
		
		drawSafetyZone(&dimensions, safetyAngle);

		GrayDBufToggleSync();
		frames++;

		safetyAngle+=5;
		if (safetyAngle >= 360)
			safetyAngle = 0;
		
	
		
		if (distance(player.x, player.y, area_width/2, area_height/2) < 32)
			player.safe = 1;
		else
			player.safe = 0;

		if (gamemode == AI && distance(aiplayer.x, aiplayer.y, area_width/2, area_height/2) < 32)
			aiplayer.safe = 1;
		else
			aiplayer.safe = 0;


		//move any free-operating projectiles
		if (OSTimerExpired(MISC_TIMER)) {
			movebombs(player.bomb, &player.numBombs);
			movebombs(aiplayer.bomb, &aiplayer.numBombs);

			decExplosions(explosion, &numExplosions);

			OSTimerRestart(MISC_TIMER);
		}

		if (OSTimerExpired(USER_TIMER)) {
			seconds++;
			if (won == -1) {
				if (player.energy < 100) player.energy += player.energyregen;	
				if (aiplayer.energy < 100) aiplayer.energy += aiplayer.energyregen;
			}
			OSTimerRestart(USER_TIMER);
			
			if (seconds % 10 == 0)
			{
				newpacket.enabled = 1;
				newpacket.x = random(area_width-10)+5;
				newpacket.y = random(area_height-10)+5;
				newpacket.type = random(5);
				newpacket.value = 1;
			}
		}

		for (i=0; i<aiplayer.numBombs; i++) {
			if (checkhit(&aiplayer.bomb[i], &player)) {
				if (numExplosions < MAX_BOMBS*2) { 
					explosion[numExplosions] = DefaultExplode(aiplayer.bomb[i].x, aiplayer.bomb[i].y);
					numExplosions++;
				}
				if (!player.safe)
					player.energy -= aiplayer.bomb[i].damage*player.shielding;

				if (gamemode == AI) {
					aiplayer.numBombs--;
					aiplayer.bomb[i] = aiplayer.bomb[aiplayer.numBombs];
					i--;
				}
			}
		}
		
		for (i=0; i<player.numBombs; i++) {
			if (checkhit(&player.bomb[i], &aiplayer)) {
				if (numExplosions < MAX_BOMBS*2) { 
					explosion[numExplosions] = DefaultExplode(player.bomb[i].x, player.bomb[i].y);
					numExplosions++;
				}
				if (gamemode == AI && !aiplayer.safe)
					aiplayer.energy -= player.bomb[i].damage*aiplayer.shielding;

				player.numBombs--;
				player.bomb[i] = player.bomb[player.numBombs];
				i--;
			}
		}

		if (gamemode == LINK)
		{
			LIO_SendData(&player, sizeof(SHIP));
			if (!LIO_RecvData(&linkPlayer, sizeof(SHIP),10))
			{
				aiplayer = linkPlayer;
			}
			else
				done = 1;
		}
		


		if (won == -1) {

			if (newpacket.enabled && distance(newpacket.x, newpacket.y, player.x, player.y) <= player.radius)
			{
				newpacket.enabled = 0;
				switch(newpacket.type)
				{
					case WEAPONUPGRADE:
						if (player.bombdamage < 5)
							player.bombdamage += newpacket.value;
						break;
					case ENERGYUPGRADE:
						player.energy = 100;
						break;
					case REGENUPGRADE:
						if (player.energyregen < 5)
							player.energyregen += newpacket.value;
						break;
					case SPEEDUPGRADE:
						if (player.speed < 16)
							player.energyregen += newpacket.value*2;
						break;
					case SHEILDUPGRADE:
						if (player.shielding > .2)
							player.shielding -= .1;
						break;
				}
			}

	
			temp = distance(player.x, player.y, aiplayer.x, aiplayer.y);
			
			if (gamemode == AI) {
				//run some AI movement routines
				if (aiplayer.energy < aiplayer.bombloss*4 || (aiplayer.safe && (aiplayer.energy < aiplayer.bombloss*30 && aiplayer.energy < 50))) {	//retreat
					aiplayer.angle = ArcTanTheta(aiplayer.x, aiplayer.y, area_width/2, area_height/2);
					if (!aiplayer.safe)
						AImovement(&aiplayer);
				}
				else {

					aiplayer.angle = ArcTanTheta(aiplayer.x, aiplayer.y, player.x, player.y);				

					if (temp > 50 || (aiplayer.safe && temp > 20)) {
						AImovement(&aiplayer);
					}
									
					if (aiplayer.numBombs < MAX_BOMBS && temp < 80 && !aiplayer.safe && !player.safe && aiplayer.energy) {
						aiplayer.bomb[aiplayer.numBombs].enabled = 1;
						aiplayer.bomb[aiplayer.numBombs].x = aiplayer.x;
						aiplayer.bomb[aiplayer.numBombs].y = aiplayer.y;
						aiplayer.bomb[aiplayer.numBombs].angle = aiplayer.angle;
						aiplayer.bomb[aiplayer.numBombs].time = 20;

						if (aiplayer.energy > aiplayer.bombloss*10 || (aiplayer.energy > aiplayer.bombloss*5 && player.energy < aiplayer.bombdamage*5))
						{
							aiplayer.bomb[aiplayer.numBombs].damage = aiplayer.bombdamage*5;
							aiplayer.energy-=aiplayer.bombloss*5;
							aiplayer.bomb[aiplayer.numBombs].radius = 3;							
						}
						else
						{
							aiplayer.bomb[aiplayer.numBombs].damage = aiplayer.bombdamage;
							aiplayer.energy-=aiplayer.bombloss;
							aiplayer.bomb[aiplayer.numBombs].radius = 1;
						}
						aiplayer.numBombs++;
					}
				
				}
				
			}
	
			//use input from link and keyboard
			keyMovement(&player);
			
			shoot = 0;
			if(_keytest(RR_2ND))
				shoot = 1;
			if((_keytest(RR_SHIFT) && TI89) || (_keytest(RR_HAND) && !TI89))
				shoot = 5;
			if (shoot != 0 && player.numBombs < MAX_BOMBS && player.energy > shoot*player.bombloss && !player.safe) {
					player.bomb[player.numBombs].enabled = 1;
					player.bomb[player.numBombs].x = player.x;
					player.bomb[player.numBombs].y = player.y;
					player.bomb[player.numBombs].angle = player.angle;
					player.bomb[player.numBombs].time = 20;
					if (shoot == 1) {
						player.bomb[player.numBombs].damage = player.bombdamage;	
						player.bomb[player.numBombs].radius = 1;
						player.energy-=player.bombloss;
					}
					else {
						player.bomb[player.numBombs].damage = player.bombdamage*5;
						player.bomb[player.numBombs].radius = 3;
						player.energy-=player.bombloss*5;
					}
					
					player.numBombs++;
			}		
			
			ReCenter(&dimensions, player.x, player.y);
		
			if (TI89 && _keytest(RR_CLEAR))
				done = 1;
	
			if (!TI89 && _keytest(RR_Q))
				done = 1;
	
		if (aiplayer.energy <= 0) {
			if (aimode == CUSTOM)
				won = 1;
			else {
				reset = 1;
				destroyed++;
			}
		}

		if (player.energy <= 0)
			won = 0;

		}
	}

	GrayOff();

	//PortRestore();
	
	free(screen);

	ClrScr();

	FontSetSys(F_8x10);

	GKeyFlush();

	if (gamemode == LINK || aimode == CUSTOM)
	{
		if (won == 1)
		{
			DrawStr(0,0,"You won!",A_NORMAL);
			DrawStr(0,50,"(Press ENTER)",A_NORMAL);
			while(ngetchx() != KEY_ENTER);
		}
	
		if (won == 0)
		{
			DrawStr(0,0,"You lost.",A_NORMAL);
			DrawStr(0,50,"(Press ENTER)",A_NORMAL);
			while(ngetchx() != KEY_ENTER);
		}
	}
	else if (won != -1)
	{
		sprintf(string, "Ships destroyed: %d", destroyed);
		DrawStr(0,0, "Game Over", A_NORMAL);
		DrawStr(0,10,string, A_NORMAL);
		DrawStr(0,50,"(Press ENTER)",A_NORMAL);
		while(ngetchx() != KEY_ENTER);
	}
	

	
	float fps = ((float)(frames))/((float)(seconds));
	
	return fps;
}

void keyMovement(SHIP *ship)
{
		if (_keytest(RR_LEFT))
			ship->angle -= 10;

		if (_keytest(RR_RIGHT))
			ship->angle += 10;
			
		if (_keytest(RR_UP)) {
			moveship(ship, ship->speed, 1);
		}

		if (_keytest(RR_DOWN)) {
			moveship(ship, ship->speed, -1);
		}
		
		if (ship->angle < 0)
			ship->angle += 360;
		if (ship->angle >= 360)
			ship->angle -= 360;	
}

void AImovement(SHIP *ship)
{
	moveship(ship, ship->speed, 1);
}


